function [B, cv, Binitial] = SRdiscrete(x, y, m0, initial, kappa);
%SR for discrete response 
%INPUT
%    m0 : specified efficient dimension
%    initial=1, if OPG is used for initial value
%            2, if OLS is used for initial value
%            0, others (a gradually reducing procedure)
%  
% OUTPUT
% B     : estimated dimension
% cv    : CV value
% Binitial: initial estimator of B.


[n,p] = size(x);
onen = ones(n,1);

mm = mean(x);
x = x - repmat(mm,n,1);
ss = inv(x'*x/n)^0.5;
x = x*ss;


m = p;
B = eye(p);
Ba = B;
BI = Ba;
B = B(:,1:m);
noip0 = 0;
noip1 = 1;
iterstop = 0;
Btmp = B;
rige = std(y)*mean(std(x,[],1));

yc = y;
x0 = x;
xc = x;

[yc, I] = sort(y);
y = y(I);
x = x(I,:);
x0 = x0(I,:);
xc = xc(I,:);
ky = [];

uy=unique(y);
for i=1:length(uy)
    yi = zeros(n,1);
    yi(y==uy(i)) = 1;
    ky = [ky yi];
end

%for i = 1:nslice
%    ends = i*floor(n/nslice);
%    if i == nslice;  
%        ends = n;
%    end
%    I = (i-1)*floor(n/nslice)+1:ends;
%    yi = zeros(n,1);
%    yi(I) = y(I)*0+1;
%    ky = [ky yi];
%end



if initial == 0
    m = p;
end
if initial == 1
    h = 1/n^(1/(p+4))*kappa;
    h2 = 2*h*h;
   	BB0 = 0;
	onexi = ones(n,p+1);
   	for j = 1:n;
        xij = x - repmat(x(j,:),n,1);
        dis = sum((xij).^2,2);
        disd = sort(dis);
        h2j = max(h2, disd(p+1));
        ker0 = (sum((xij).^2,2))/h2j;
      	ker = exp(-ker0);
   		onexi(:,1:p) = xij;
   		xk = (onexi.*repmat(ker, 1, p+1))';
   		abi = inv(xk*onexi+eye(p+1)/n)*xk*ky;        
        BB0 = BB0 + abi(1:p,:)*abi(1:p,:)';
    end   
    [B, R] = eig(BB0/n);
    Binitial = B(:,p-m0+1:p);  
    B0 = Binitial;
    m = m0;
    if p > 10
        B0 = B(:,p-9:p);
        m = 10;
    end
    Ba = B0;
    B = B0;
end

BB0 = 0;
if initial == 2
    m = m0;
    for i = 1:nslice
        ends = i*floor(n/nslice);
        if i == nslice;  
            ends = n;
        end
        I = (i-1)*floor(n/nslice)+1:ends;
        xi = [x(I,:) ones(length(I),1)];
        ncol=length(xi(1,:));D=1.0E-6*eye(ncol);    %ridge effect added by Hanson
        bi = inv(xi'*xi+D)*(xi'*y(I));              %ridge effect added by Hanson
        BB0 = BB0 + bi(1:p)*bi(1:p)';
    end   
    [B, R] = eig(BB0/n);
    B0 = Binitial;
    m = m0;
    if p > 10
        B0 = B(:,p-9:p);
        m = 10;
    end
    Ba = B0;
    B = B0;
end

cv = 0;
Binitial = [];
if (initial ==1)+(initial ==2)
B0 = ss*B0;
for i = 1:size(Binitial,2);
    Binitial(:,i) = Binitial(:,i)/sqrt(Binitial(:,i)'*Binitial(:,i));
end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

niter = floor(p*3/2);
for iter = 1:niter;
    x = xc;
    if iter >= p;
        x = x0;
    end
  	adj = p^(1/iter)*n^(1/(m0+4)-1/(m+4));
    n1 = size(ky,2);
    h = p^(1/(iter+1))*mean(std(x*Ba,[],1))/n^(1/(m+4));
    h2 = 2*h*h*adj;
   
   ABI = zeros(m, n*n); 
   for iiter = 1:max(1, (m < p)*floor(m/2));
	dd = zeros(m*p, m*p);
   	dc = zeros(m*p,1);
   	for j = 1:n;
        xij = x - repmat(x(j,:),n,1);
        dis = sum((xij*Ba).^2,2);
        disd = sort(dis);
        h2j = max(h2, disd(p+1));
        sxij = dis + sum(xij.^2,2)/1.5^iter;
      	ker = exp(-sxij/h2j);  
        rker = repmat(ker, 1, p+1);
   		onexi = [xij*B onen];
   		xk = (onexi.*rker(:, 1:m+1))';
   		abi = inv(xk*onexi+eye(m+1)/n)*(xk*ky);
      
      	kxij = (xij.*rker(:,1:p))';
      	kxijy = kxij*(ky - repmat(abi(m+1,:),n,1));
      	ddx = kxij*xij;
      	for k1 = 1:m
         	ka = (k1-1)*p+1:k1*p;
         	dc(ka) = dc(ka) + kxijy*abi(k1,:)';
      	end
        tmp = abi(1:m, :)*abi(1:m, :)';
        dd = dd + kron(tmp, ddx);
      
        ABI(:, (j-1)*n1+1:j*n1) = abi(1:m,:); 
   	end
    
	   B = inv(dd+ rige*eye(length(dc))/n)*dc;
       
   	   B0 = reshape(B, p, m)*ABI;
       [B, R] = eig(B0*B0');
       B = B(:,p-m+1:p);
       Ba = B;
       if (max(svd(B*B'-BI*BI')) < 0.001);
           break
       end
       BI = B;
       
   end

   mb = m;
   ma = min(max(m-1,m0), 10);
   m = ma; 
   B = B(:,mb-m+1:mb);
   Ba = B;
   if (max(svd(B*B'-BI*BI')) < 0.001)*(iter>p+3);
       break
   end
   BI = Ba;   
%   [iter,max(svd(B*B'-BI*BI'))]
end

cv = 0;
kye = ky;
	for j = 1:n;
        xij = x - repmat(x(j,:),n,1);
        sxij = sum((xij*Ba).^2,2) + 0*sum(xij.^2,2)/1.5^iter;
        ker = exp(-sxij/h2);  
        ker(j) = 0;
        if mean(ker) > 1/n/n;
            kye(j,:) = ker'*ky/sum(ker);
        end      
  end   
cv = (ky-kye).^2;
cv = sum(sum(cv))/sum(sum(cv>0));


B = ss*B;
for i = 1:size(B,2);
    B(:,i) = B(:,i)/sqrt(B(:,i)'*B(:,i));
end


