function [out] =covariateinf(x,y,numslice)
% [out] =covariateinf(x,y,H) returns covariance information matrix using
%the sliced method for J_{x,y} by mimicing the discrete y version. 
% Each row of x and y represent an observation. 
%Reference: Lindsay and Yao
% "Fisher Discrimination Matrix and Dimension Reduction".

%%%%Example %%%%%%%%%
%n=400;p=10;d=2;
%true1=[1,zeros(1,p-1)]';true2=[0,1,zeros(1,p-2)]';
%x=normrnd(0,1,n,p);e=normrnd(0,1,n,1);
%y=cos(2*x*true1)-cos(x*true2)+0.5*e;
%out=covariateinf1(x,y,3);
%out.direction[:,1:d]  returns the found d directions.  
 
 
s=size(x);d=s(2);n=s(1);svx=cov(x)^(1/2);
 
if exist('numslice')==0
    numslice=10;   
end
[sy,ind]=sort(y);numrep=round(n/numslice);
infxy=0;
for i=1:numslice-1
    xs=x(ind((i-1)*numrep+1:i*numrep),:);
    infxy=infxy+numrep/n*fisherinf(xs);
end
xs=x(ind((numslice-1)*numrep+1:n),:);
infxy=infxy+(n-(numslice-1)*numrep)/n*fisherinf(xs);
infx=fisherinf(x);
covxinf=infxy-infx;
[sv,sd]=eig(covxinf*cov(x)); 
temp=diag(sd);[a,b]=sort(-temp);sd=temp(b);sv=sv(:,b);
 
out.direction=sv;
out.eigenvalue=sd';
out.covariateinf=covxinf;
out.infxy=infxy;
out.infx=infx;
 
 
 
function [out]=fisherinf(x,H)
%[rootvf2,estinf] = wna(x,H) returns the squre root of covariance matrix of
%density square of x and the fisher informatino for f2
%regular bandwidth
s=size(x);N=s(1);d=s(2);
if exist('H')==0
    if d>N/2 
        H=diag(std(x));
    else
        H=cov(x)^(1/2);
    end
    H=H*(4/(d+2)/N)^(1/(d+4));
end
H2=H*H;invH2=inv(H2);
u0=0;u1=zeros(d);u2=zeros(d);u3=zeros(1,d);

%Based on the symmetry of xi and xj, we only need to calculate       %the cases when i<j and multiple them by 2 and then plus the    %case i=j 

for i=1:(N-1);
    for j=(i+1):N;
        %a=mvnpdf(x(i,:),x(j,:),2*H2);
        temp=x(i,:)-x(j,:);
        a=exp(-0.25*temp*invH2*temp'); %a is the kernel density 
        u0=u0+a*2; %u0 calculates the second equation at p131
        u1=u1+2*a*(1/2*invH2-invH2*((x(i,:)-x(j,:))'*(x(i,:)-x(j,:)))*invH2/4); %u1 calculates third equation at p131
    end
   temp=x(i,:)-x(i,:);a=exp(-0.25*temp*invH2*temp');
   u0=u0+a; u1=u1+a*(1/2*invH2); %the case of i=j
end
temp=x(N,:)-x(N,:);a=exp(-0.25*temp*invH2*temp');
u0=u0+a;u1=u1+a*(1/2*invH2);
u0=u0/N^2;u1=u1/N^2;
out=u1/u0; %The fisher information for density square transformation f2 without standarization (using vf2)
 
 
 
 

